from . import *
from shutil import copy
import os

class Command(ParsingBaseCommand):
    help = 'creates a boilerplate webpack configuration file for your next webpack project.'

    def parse(self, parser, argv):
        args, unknown = parser.parse_known_args(argv)
        self.unknown_arguments = unknown
        return args

    def add_arguments(self, parser):
        parser.add_argument('config_name',
                            help='the full name of the configuration file (ending in js)')

    def handle(self, *args, **options):
        from django.apps import apps
        copy(os.path.join(BASEDIR, 'webpack.config.js'), options['config_name'])



from . import *
from django.core.exceptions import ImproperlyConfigured
from subprocess import call, CalledProcessError

class Command(ParsingBaseCommand):
    help = 'A wrapper for running npm install on a django app. Any arguments that are '\
           'not captured by the command is passed unaltered to npm'

    def parse(self, parser, argv):
        args, unknown = parser.parse_known_args(argv)
        self.unknown_arguments = unknown
        return args

    def add_arguments(self, parser):
        parser.add_argument('app',
                            help='the app for which to install dependencies')

    def npm_install_app(self, settings, appconfig):
        try:
            with env(NODE_PATH=settings.NODE_MODULES):
                with cd(settings.PROJECT_BASE):
                    call(['npm', 'install', appconfig.path] + self.unknown_arguments)
        except CalledProcessError as e:
            print('Are you sure Node Package Manager (NPM) is installed?')
            raise

    def handle(self, *args, **options):
        from django.apps import apps
        from webpack_helper.conf import settings
        self.npm_install_app(settings, apps.get_app_config(options['app']))



import sys
import os
from django.core.management.base import BaseCommand, CommandError
import django.core.management.base as base_management
from contextlib import contextmanager

__all__ = ['cd', 'env', 'mkdirs', 'ParsingBaseCommand', 'BASEDIR']
BASEDIR = os.path.dirname(os.path.abspath(__file__))

@contextmanager
def cd(path):
    old = os.getcwd()
    try:
        os.chdir(path)
        yield
    finally:
        os.chdir(old)


@contextmanager
def env(env=None, **kwargs):
    env = env or {}
    env.update(kwargs)

    old_env = dict(os.environ)
    try:
        os.environ.update(env)
        yield
    finally:
        os.environ.clear()
        os.environ.update(old_env)


def mkdirs(path):
    if not os.path.exists(path):
        os.makedirs(path)

# see https://docs.djangoproject.com/en/1.9/howto/custom-management-commands/#ref-basecommand-subclasses
class ParsingBaseCommand(BaseCommand):
    """
    Allows us to intercept parsing of commandline arguments
    """
    def parse(self, parser, argv):
        return parser.parse_args(argv[2:])

    def run_from_argv(self, argv):
        """
        Set up any environment changes requested (e.g., Python path
        and Django settings), then run this command. If the
        command raises a ``CommandError``, intercept it and print it sensibly
        to stderr. If the ``--traceback`` option is present or the raised
        ``Exception`` is not ``CommandError``, raise it.
        """
        self._called_from_command_line = True
        parser = self.create_parser(argv[0], argv[1])

        if self.use_argparse:
            options = self.parse(parser, argv[2:])
            cmd_options = vars(options)
            # Move positional args out of options to mimic legacy optparse
            args = cmd_options.pop('args', ())
        else:
            options, args = self.parse(parser, argv[2:])
            cmd_options = vars(options)
        base_management.handle_default_options(options)
        try:
            self.execute(*args, **cmd_options)
        except Exception as e:
            if options.traceback or not isinstance(e, CommandError):
                raise
            # SystemCheckError takes care of its own formatting.
            if isinstance(e, base_management.SystemCheckError):
                self.stderr.write(str(e), lambda x: x)
            else:
                self.stderr.write('%s: %s' % (e.__class__.__name__, e))
            sys.exit(1)
        finally:
            base_management.connections.close_all()



from setuptools import setup, find_packages

setup(
    name="django-webpack-helper",
    version="0.1.3",
    url="http://github.com/chriscz/django-webpack-helper",
    description="A django app that helps integrate webpack",
    author="Chris Coetzee",
    license='Mozilla Public License 2.0 (MPL 2.0)',
    author_email='chriscz93@gmail.com',
    packages=find_packages(exclude=['exampleapp']),
    install_requires=[
        'django-webpack-loader'
    ],
    include_package_data=True,

    classifiers = [
        "Programming Language :: Python :: 2.7",            
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
    ]
)



import os
import json
from subprocess import call

from . import *

class Command(ParsingBaseCommand):
    help = 'executes webpack to build project assets (JS/CSS). Any arguments that are '\
           'not captured by the command is passed unaltered to webpack'

    def parse(self, parser, argv):
        args, unknown = parser.parse_known_args(argv)
        self.unknown_arguments = unknown
        return args

    def add_arguments(self, parser):
        parser.add_argument('applabel',
                            help='the app for which to compile webpack assets')

        parser.add_argument('--webpack-help',
                            help="executes webpack's help command (so as not to interfere with django's)",
                            action='store_true')

    def handle(self, *args, **options):
        from django.apps import apps
        from webpack_helper.conf import settings

        # --- load django settings settings
        NODE_BIN = os.path.join(settings.NODE_MODULES, '.bin')

        # ensure paths exist
        mkdirs(settings.CONFIG_DIR)
        mkdirs(settings.STATS_DIR)
        mkdirs(settings.BUNDLE_DIR)

        # --- build app specific path
        app_label = options['applabel']
        WEBPACK_CONF_FILE = os.path.join(settings.CONFIG_DIR, app_label + '.json')
        WEBPACK_STATS_FILE = os.path.join(settings.STATS_DIR, app_label + '.json')

        app_config = apps.get_app_config(app_label)

        # --- generate a config file
        config = {}
        config['WEBPACK_EXTRA_MODULE_DIRECTORIES'] = [settings.NODE_MODULES]
        config['WEBPACK_PUBLIC_PATH'] = settings.PUBLIC_PATH
        config['WEBPACK_BUNDLE_DIR'] = settings.BUNDLE_DIR
        config['WEBPACK_STATS_FILE'] = WEBPACK_STATS_FILE

        with open(WEBPACK_CONF_FILE, 'w') as config_file:
            json.dump(config, config_file, indent=4)

        # --- prepare arguments for execution
        webpack = os.path.join(NODE_BIN, 'webpack')
        args = [webpack] + self.unknown_arguments
        if options['webpack_help']:
            args.append('--help')

        # --- prepare environment & call webpack
        with env(NODE_PATH=settings.NODE_MODULES,
                 DJANGO_WEBPACK_CONFIG=WEBPACK_CONF_FILE):
            with cd(app_config.path):
                    call(args)



import sys
import os
import re

from django.conf import settings as django_settings
from django.core.exceptions import ImproperlyConfigured


class AttrView(object):
    def __init__(self, dictionary):
        self.__dictionary = dictionary

    def __contains__(self, key):
        return key in self.__dictionary

    def __setattr__(self, key, value):
        if key == '_' + type(self).__name__ + '__dictionary':
            object.__setattr__(self, key, value)
        else:
            self.__dictionary[key] = value

    def __getattr__(self, key):
        return self.__dictionary[key]

    def __iter__(self):
        for key in self.__dict__:
            yield key

    def __len__(self):
        return len(self.__dictionary)

    def __str__(self):
        return str('<AttrView '+str(self.__dictionary) + '>')


def gets(name, *default):
    if len(default) > 1:
        raise RuntimeError("Only a single default is allowed")
    return getattr(django_settings, name, *default)

PROJECT_ROOT = gets("BASE_DIR")

static_url = gets('STATIC_URL')
if not static_url.endswith('/'):
    static_url += '/'

defaults = {
   'NODE_MODULES': 'node_modules',
   'BASE_DIR': 'webpack',
   'BUNDLE_DIR': 'bundles/',
   'CONFIG_DIR': 'config/',
   'STATS_DIR': 'stats/',
   'STATIC_DIR': 'static/',
   'PUBLIC_PATH_BASE': gets('STATIC_URL')
}

settings_pre = dict(defaults)
settings_pre.update(getattr(django_settings, 'WEBPACK_HELPER', {}))

# --- check that setting names are correct
for key in settings_pre:
    if key not in defaults:
        msg = 'django-webpack-helper: setting %s is not a recognized setting'
        raise ImproperlyConfigured(msg % key)

# --- ensure that all paths are absolute
settingsdict = {}

pre = AttrView(settings_pre)
settings = AttrView(settingsdict)

def relto_or_abs(path, relto=PROJECT_ROOT):
    """
    if path is not absolute, return it relative to relto
    else keep it as is.
    """
    if not os.path.isabs(path):
        return os.path.join(relto, path)
    return path

def relto(root, path, name):
    if os.path.isabs(path):
        msg = 'django-webpack-helper: setting %s should be a relative path, but got %s.'
        raise ImproperlyConfigured(msg % (name, path))
    return os.path.join(root, path)


# --- make all paths absolute
settings.PROJECT_BASE = PROJECT_ROOT

settings.NODE_MODULES = relto_or_abs(pre.NODE_MODULES)
settings.BASE_DIR = relto_or_abs(pre.BASE_DIR)

settings.CONFIG_DIR = relto(settings.BASE_DIR, pre.CONFIG_DIR, 'CONFIG_DIR')
settings.STATS_DIR = relto(settings.BASE_DIR, pre.STATS_DIR, 'STATS_DIR')
settings.STATIC_DIR = relto(settings.BASE_DIR, pre.STATIC_DIR, 'STATIC_DIR')
settings.BUNDLE_DIR = relto(settings.STATIC_DIR, pre.BUNDLE_DIR, 'BUNDLE_DIR')

settings.PUBLIC_PATH = pre.PUBLIC_PATH_BASE + '/' + pre.BUNDLE_DIR + '/'
settings.PUBLIC_PATH = re.sub(r'/{2,}', '/', settings.PUBLIC_PATH)
